@@ -54,7 +54,7 @@ from hypothesis.reporting import report, verbose_report, current_verbosity
 from hypothesis.statistics import note_engine_for_statistics
 from hypothesis.internal.compat import ceil, hbytes, qualname, \
     str_to_bytes, benchmark_time, get_type_hints, getfullargspec, \
-    encoded_filepath, bad_django_TestCase
+    int_from_bytes, encoded_filepath, bad_django_TestCase
 from hypothesis.internal.coverage import IN_COVERAGE_TESTS
 from hypothesis.utils.conventions import infer, not_set
 from hypothesis.internal.escalation import is_hypothesis_file, \
@@ -115,8 +115,10 @@ def seed(seed):
     May be any hashable object. No exact meaning for seed is provided
     other than that for a fixed seed value Hypothesis will try the same
     actions (insofar as it can given external sources of non-
-    determinism. e.g. timing and hash randomization). Overrides the
-    derandomize setting if it is present.
+    determinism. e.g. timing and hash randomization).
+
+    Overrides the derandomize setting, which is designed to enable
+    deterministic builds rather than reproducing observed failures.
 
     """
     def accept(test):
@@ -298,7 +300,7 @@ def get_random_for_wrapped_test(test, wrapped_test):
         return Random(
             wrapped_test._hypothesis_internal_use_seed)
     elif settings.derandomize:
-        return Random(function_digest(test))
+        return Random(int_from_bytes(function_digest(test)))
     elif global_force_seed is not None:
         return Random(global_force_seed)
     else:
@@ -448,7 +450,6 @@ class StateForActualGivenExecution(object):
         self.test_runner = test_runner
         self.search_strategy = search_strategy
         self.settings = settings
-        self.at_least_one_success = False
         self.last_exception = None
         self.falsifying_examples = ()
         self.__was_flaky = False
@@ -676,12 +677,11 @@ class StateForActualGivenExecution(object):
                             arc(filename, source, target)
                             for source, target in covdata.arcs(filename)
                         )
-            if result is not None and self.settings.perform_health_check:
+            if result is not None:
                 fail_health_check(self.settings, (
                     'Tests run under @given should return None, but '
                     '%s returned %r instead.'
                 ) % (self.test.__name__, result), HealthCheck.return_value)
-            self.at_least_one_success = True
             return False
         except UnsatisfiedAssumption:
             data.mark_invalid()
@@ -774,31 +774,20 @@ class StateForActualGivenExecution(object):
                 ) % (
                     self.settings.timeout, runner.valid_examples),
                     self.settings)
-            if runner.valid_examples < min(
-                self.settings.min_satisfying_examples,
-                self.settings.max_examples,
-            ) and not (
-                runner.exit_reason == ExitReason.finished and
-                self.at_least_one_success
-            ):
+            if runner.valid_examples == 0:
                 if timed_out:
                     raise Timeout((
                         'Ran out of time before finding a satisfying '
-                        'example for '
-                        '%s. Only found %d examples in ' +
-                        '%.2fs.'
+                        'example for %s. Only found %d examples in %.2fs.'
                     ) % (
                         get_pretty_function_description(self.test),
                         runner.valid_examples, run_time
                     ))
                 else:
-                    raise Unsatisfiable((
-                        'Unable to satisfy assumptions of hypothesis '
-                        '%s. Only %d examples considered '
-                        'satisfied assumptions'
-                    ) % (
-                        get_pretty_function_description(self.test),
-                        runner.valid_examples,))
+                    raise Unsatisfiable(
+                        'Unable to satisfy assumptions of hypothesis %s.' %
+                        (get_pretty_function_description(self.test),)
+                    )
 
         if not self.falsifying_examples:
             return
@@ -907,11 +896,14 @@ def given(*given_arguments, **given_kwargs):
     This is the main entry point to Hypothesis.
     """
     def run_test_with_generator(test):
+        if hasattr(test, '_hypothesis_internal_test_function_without_warning'):
+            # Pull out the original test function to avoid the warning we
+            # stuck in about using @settings without @given.
+            test = test._hypothesis_internal_test_function_without_warning
         if inspect.isclass(test):
             # Provide a meaningful error to users, instead of exceptions from
             # internals that assume we're dealing with a function.
             raise InvalidArgument('@given cannot be applied to a class.')
-
         generator_arguments = tuple(given_arguments)
         generator_kwargs = dict(given_kwargs)
 
@@ -939,15 +931,15 @@ def given(*given_arguments, **given_kwargs):
             __tracebackhide__ = True
 
             if getattr(test, 'is_hypothesis_test', False):
-                note_deprecation((
+                note_deprecation(
                     'You have applied @given to a test more than once. In '
                     'future this will be an error. Applying @given twice '
                     'wraps the test twice, which can be extremely slow. A '
                     'similar effect can be gained by combining the arguments '
-                    'to the two calls to given. For example, instead of '
+                    'of the two calls to given. For example, instead of '
                     '@given(booleans()) @given(integers()), you could write '
                     '@given(booleans(), integers())'
-                ))
+                )
 
             settings = wrapped_test._hypothesis_internal_use_settings
 
@@ -1067,6 +1059,9 @@ def given(*given_arguments, **given_kwargs):
             if not (attrib.startswith('_') or hasattr(wrapped_test, attrib)):
                 setattr(wrapped_test, attrib, getattr(test, attrib))
         wrapped_test.is_hypothesis_test = True
+        if hasattr(test, '_hypothesis_internal_settings_applied'):
+            # Used to check if @settings is applied twice.
+            wrapped_test._hypothesis_internal_settings_applied = True
         wrapped_test._hypothesis_internal_use_seed = getattr(
             test, '_hypothesis_internal_use_seed', None
         )
@@ -1085,10 +1080,9 @@ def find(specifier, condition, settings=None, random=None, database_key=None):
     matches the predicate function ``condition``."""
     settings = settings or Settings(
         max_examples=2000,
-        min_satisfying_examples=0,
         max_shrinks=2000,
     )
-    settings = Settings(settings, perform_health_check=False)
+    settings = Settings(settings, suppress_health_check=HealthCheck.all())
 
     if database_key is None and settings.database is not None:
         database_key = function_digest(condition)
@@ -1105,6 +1099,7 @@ def find(specifier, condition, settings=None, random=None, database_key=None):
     random = random or new_random()
     successful_examples = [0]
     last_data = [None]
+    last_repr = [None]
 
     def template_condition(data):
         with BuildContext(data):
@@ -1119,24 +1114,21 @@ def find(specifier, condition, settings=None, random=None, database_key=None):
         if success:
             successful_examples[0] += 1
 
-        if settings.verbosity == Verbosity.verbose:
+        if settings.verbosity >= Verbosity.verbose:
             if not successful_examples[0]:
-                report(lambda: u'Trying example %s' % (
-                    nicerepr(result),
-                ))
+                report(
+                    u'Tried non-satisfying example %s' % (nicerepr(result),))
             elif success:
                 if successful_examples[0] == 1:
-                    report(lambda: u'Found satisfying example %s' % (
-                        nicerepr(result),
-                    ))
+                    last_repr[0] = nicerepr(result)
+                    report(u'Found satisfying example %s' % (last_repr[0],))
                     last_data[0] = data
                 elif (
                     sort_key(hbytes(data.buffer)) <
                     sort_key(last_data[0].buffer)
-                ):
-                    report(lambda: u'Shrunk example to %s' % (
-                        nicerepr(result),
-                    ))
+                ) and nicerepr(result) != last_repr[0]:
+                    last_repr[0] = nicerepr(result)
+                    report(u'Shrunk example to %s' % (last_repr[0],))
                     last_data[0] = data
         if success and not data.frozen:
             data.mark_interesting()
@@ -1153,12 +1145,11 @@ def find(specifier, condition, settings=None, random=None, database_key=None):
             list(runner.interesting_examples.values())[0].buffer)
         with BuildContext(data):
             return data.draw(search)
-    if (
-        runner.valid_examples <= settings.min_satisfying_examples and
+    if runner.valid_examples == 0 and (
         runner.exit_reason != ExitReason.finished
     ):
         if settings.timeout > 0 and run_time > settings.timeout:
-            raise Timeout((
+            raise Timeout((  # pragma: no cover
                 'Ran out of time before finding enough valid examples for '
                 '%s. Only %d valid examples found in %.2f seconds.'
             ) % (
@@ -1166,11 +1157,9 @@ def find(specifier, condition, settings=None, random=None, database_key=None):
                 runner.valid_examples, run_time))
 
         else:
-            raise Unsatisfiable((
-                'Unable to satisfy assumptions of '
-                '%s. Only %d examples considered satisfied assumptions'
-            ) % (
-                get_pretty_function_description(condition),
-                runner.valid_examples,))
+            raise Unsatisfiable(
+                'Unable to satisfy assumptions of %s.' %
+                (get_pretty_function_description(condition),)
+            )
 
     raise NoSuchExample(get_pretty_function_description(condition))
